@@ -20,27 +20,11 @@
  *
  **************************************************************************/
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
-#include <string.h>
-#include <sys/types.h>
-#include <stdlib.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <unistd.h>
-#include <errno.h>
-
-#include <glib.h>
-#include <glib/gi18n-lib.h>
-
-#include <nm-connection.h>
-#include <nm-setting-vpn.h>
+#include "nm-default.h"
 
 #include "advanced-dialog.h"
-#include "nm-pptp.h"
-#include "../src/nm-pptp-service.h"
+
+#include <errno.h>
 
 #define COL_NAME  0
 #define COL_VALUE 1
@@ -68,6 +52,7 @@ static const char *advanced_keys[] = {
 	NM_PPTP_KEY_NO_VJ_COMP,
 	NM_PPTP_KEY_LCP_ECHO_FAILURE,
 	NM_PPTP_KEY_LCP_ECHO_INTERVAL,
+	NM_PPTP_KEY_UNIT_NUM,
 	NULL
 };
 
@@ -89,11 +74,11 @@ advanced_dialog_new_hash_from_connection (NMConnection *connection,
                                           GError **error)
 {
 	GHashTable *hash;
-	NMSettingVPN *s_vpn;
+	NMSettingVpn *s_vpn;
 
 	hash = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
 
-	s_vpn = (NMSettingVPN *) nm_connection_get_setting (connection, NM_TYPE_SETTING_VPN);
+	s_vpn = nm_connection_get_setting_vpn (connection);
 	nm_setting_vpn_foreach_data_item (s_vpn, copy_values, hash);
 	return hash;
 }
@@ -103,25 +88,23 @@ handle_mppe_changed (GtkWidget *check, gboolean is_init, GtkBuilder *builder)
 {
 	GtkWidget *widget;
 	gboolean use_mppe;
+	gboolean mppe_sensitive;
 	GtkTreeModel *model;
 	GtkTreeIter iter;
 	gboolean valid;
 
+	mppe_sensitive = gtk_widget_get_sensitive (check);
 	use_mppe = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (check));
 
 	/* (De)-sensitize MPPE related stuff */
 	widget = GTK_WIDGET (gtk_builder_get_object (builder, "ppp_mppe_security_label"));
-	gtk_widget_set_sensitive (widget, use_mppe);
+	gtk_widget_set_sensitive (widget, use_mppe && mppe_sensitive);
 
 	widget = GTK_WIDGET (gtk_builder_get_object (builder, "ppp_mppe_security_combo"));
-	if (!use_mppe)
-		gtk_combo_box_set_active (GTK_COMBO_BOX (widget), 0); /* default */
-	gtk_widget_set_sensitive (widget, use_mppe);
+	gtk_widget_set_sensitive (widget, use_mppe && mppe_sensitive);
 
 	widget = GTK_WIDGET (gtk_builder_get_object (builder, "ppp_allow_stateful_mppe"));
-	if (!use_mppe)
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), FALSE);
-	gtk_widget_set_sensitive (widget, use_mppe);
+	gtk_widget_set_sensitive (widget, use_mppe && mppe_sensitive);
 
 	/* At dialog-setup time, don't touch the auth methods if MPPE is disabled
 	 * since that could overwrite the user's previously chosen auth methods.
@@ -145,8 +128,7 @@ handle_mppe_changed (GtkWidget *check, gboolean is_init, GtkBuilder *builder)
 		case TAG_PAP:
 		case TAG_CHAP:
 		case TAG_EAP:
-			gtk_list_store_set (GTK_LIST_STORE (model), &iter, COL_VALUE, !use_mppe, -1);
-			gtk_list_store_set (GTK_LIST_STORE (model), &iter, COL_SENSITIVE, !use_mppe, -1);
+			gtk_list_store_set (GTK_LIST_STORE (model), &iter, COL_SENSITIVE, !(use_mppe && mppe_sensitive), -1);
 			break;
 		default:
 			break;
@@ -257,13 +239,8 @@ check_toggled_cb (GtkCellRendererToggle *cell, gchar *path_str, gpointer user_da
 	}
 	/* Make sure MPPE is non-sensitive if MSCHAP and MSCHAPv2 are disabled */
 	widget = GTK_WIDGET (gtk_builder_get_object (builder, "ppp_use_mppe"));
-	if (!mschap_state && !mschap2_state) {
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), FALSE);
-		gtk_widget_set_sensitive (widget, FALSE);
-		/* Make sure also MPPE security combo and stateful checkbox are non-sensitive */
-		mppe_toggled_cb (widget, builder);
-	} else
-		gtk_widget_set_sensitive (widget, TRUE);
+	gtk_widget_set_sensitive (widget, mschap_state || mschap2_state);
+	mppe_toggled_cb (widget, builder);
 }
 
 static void
@@ -288,7 +265,7 @@ auth_methods_setup (GtkBuilder *builder, GHashTable *hash)
 	value = g_hash_table_lookup (hash, NM_PPTP_KEY_REQUIRE_MPPE);
 	if (value && !strcmp (value, "yes"))
 		use_mppe = TRUE;
-	
+
 	/* Or MPPE-128 */
 	value = g_hash_table_lookup (hash, NM_PPTP_KEY_REQUIRE_MPPE_128);
 	if (value && !strcmp (value, "yes"))
@@ -398,13 +375,21 @@ auth_methods_setup (GtkBuilder *builder, GHashTable *hash)
 		gtk_widget_set_sensitive (widget, TRUE);
 }
 
+static void
+checkbox_toggled_update_widget_cb (GtkWidget *check, gpointer user_data)
+{
+	GtkWidget *widget = (GtkWidget*) user_data;
+
+	gtk_widget_set_sensitive (widget, gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (check)));
+}
+
 GtkWidget *
 advanced_dialog_new (GHashTable *hash)
 {
 	GtkBuilder *builder;
 	GtkWidget *dialog = NULL;
 	char *ui_file = NULL;
-	GtkWidget *widget;
+	GtkWidget *widget, *spin;
 	const char *value;
 	gboolean mppe = FALSE;
 	GError *error = NULL;
@@ -491,6 +476,31 @@ advanced_dialog_new (GHashTable *hash)
 	handle_mppe_changed (widget, TRUE, builder);
 	g_signal_connect (G_OBJECT (widget), "toggled", G_CALLBACK (mppe_toggled_cb), builder);
 
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "ppp_unit_checkbutton"));
+	spin = GTK_WIDGET (gtk_builder_get_object (builder, "ppp_unit_spinbutton"));
+	g_signal_connect (G_OBJECT (widget), "toggled", G_CALLBACK (checkbox_toggled_update_widget_cb), spin);
+
+	value = g_hash_table_lookup (hash, NM_PPTP_KEY_UNIT_NUM);
+	if (value && *value) {
+		long int tmp;
+
+		errno = 0;
+		tmp = strtol (value, NULL, 10);
+		if (errno == 0 && tmp >= 0 && tmp < 65536) {
+			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), TRUE);
+
+			widget = GTK_WIDGET (gtk_builder_get_object (builder, "ppp_unit_spinbutton"));
+			gtk_spin_button_set_value (GTK_SPIN_BUTTON (widget), (gdouble) tmp);
+			gtk_widget_set_sensitive (widget, TRUE);
+		}
+	} else {
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), FALSE);
+
+		widget = GTK_WIDGET (gtk_builder_get_object (builder, "ppp_unit_spinbutton"));
+		gtk_spin_button_set_value (GTK_SPIN_BUTTON (widget), 0.0);
+		gtk_widget_set_sensitive (widget, FALSE);
+	}
+
 out:
 	g_free (ui_file);
 	return dialog;
@@ -590,6 +600,16 @@ advanced_dialog_new_hash_from_dialog (GtkWidget *dialog, GError **error)
 		valid = gtk_tree_model_iter_next (model, &iter);
 	}
 
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "ppp_unit_checkbutton"));
+	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget))) {
+		int unit_num;
+
+		widget = GTK_WIDGET (gtk_builder_get_object (builder, "ppp_unit_spinbutton"));
+		unit_num = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (widget));
+		g_hash_table_insert (hash, g_strdup (NM_PPTP_KEY_UNIT_NUM),
+		                     g_strdup_printf ("%d", unit_num));
+	}
+
 	return hash;
 }
 
